package ics332.deadlock.a_spinlocks;

/**
 * Thread that acquires lock 1 then lock 2 works and release
 * This is an educative example to demonstrate how deadlocks happen
 * 
 * The thread voluntarily is set asleep before trying to acquire 2
 *  
 * Uses a spin lock
 * 
 */
public class Thread12 extends Thread {
	private Resources resources;

	public Thread12(Resources resources) {
		this.resources = resources;
	}
	
	@Override
	public void run() {
		try {
			System.out.println(Thread12.class.getName() + " running");
			while (!this.resources.lock1.tryLock()) { // Spin lock 
				;
			}
			System.out.println(Thread12.class.getName() + " lock1 acquired");
			Thread.sleep(10);
			while (!this.resources.lock2.tryLock()) { // Spin lock
				;
			}
			System.out.println(Thread12.class.getName() + " lock2 acquired");
			Thread.sleep(10);
			System.out.println(Thread12.class.getName() + " working");
			Thread.sleep(1000);
			this.resources.lock2.unlock();
			System.out.println(Thread12.class.getName() + " lock2 released");
			Thread.sleep(10);
			this.resources.lock1.unlock();
			System.out.println(Thread12.class.getName() + " lock1 released");
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}
}
